/*
 * This Source Code is subject to the terms of the Mozilla Public License
 * version 2.0 (the "License"). You can obtain a copy of the License at
 * http://mozilla.org/MPL/2.0/.
 *
 * The Original Code is Forward Extension.
 * The Initial Developer of the Original Code is Onno Ekker.
 * Portions created by Onno Ekker <o.e.ekker@gmail.com> are Copyright
 * (c) 2008. All Rights Reserved.   
 */

var myForwardExtension = {

  myLogToErrorConsole: function(aMessage, aSourceName, aSourceLine, aLineNumber,
                                aColumnNumber, aFlags, aCategory)
  {
    var consoleService = Components.classes['@mozilla.org/consoleservice;1'].
                                    getService(Components.interfaces.nsIConsoleService);
    if (aFlags === null)
    {
      consoleService.logStringMessage(aMessage);
    }
    else
    {
      var scriptError = Components.classes['@mozilla.org/scripterror;1'].
                                   createInstance(Components.interfaces.nsIScriptError);
      scriptError.init(aMessage, aSourceName, aSourceLine, aLineNumber,
                       aColumnNumber, aFlags, aCategory);
      consoleService.logMessage(scriptError);
    }
  },

  // forward-button

  updateForwardButtonMenu: function(event)
  {
    var forwardType = 0;
    var forwardInlineMenuItem = document.getElementById('button-forwardAsMenu-forwardInline');
    var forwardAsAttachmentMenuItem = document.getElementById('button-forwardAsMenu-forwardAsAttachment');
    var strbundle = document.getElementById('bundle_forward');

    function ucfirst(menuItem)
    {
      var origLabel = menuItem.getAttribute('label');
      var newLabel = origLabel.replace(/^(\S)/, function(t, a) { return a.toLocaleUpperCase(); });
      if (newLabel !== origLabel)
      {
        menuItem.setAttribute('label', newLabel);
      }
    }

    // mail.forward_message_mode could be 1, if the user migrated from 4.x
    // 1 (forward as quoted) is obsolete, so we treat it as forward inline
    // since that is more like forward as quoted then forward as attachment
    try
    {
      forwardType = gPrefBranch.getIntPref('mail.forward_message_mode');
    }
    catch (ex)
    {
      myLogToErrorConsole(ex, 'chrome://forward/content/forward.js',
                          '    forwardType = gPrefBranch.getIntPref(\'mail.forward_message_mode\');',
                          74, 0, 2, 'javascript');
      forwardType = 2; // Let's make Inline the default
    }

    if (forwardInlineMenuItem)
    {
      forwardInlineMenuItem.setAttribute('default', (forwardType !== 0));
      try
      {
        forwardInlineMenuItem.setAttribute('label', strbundle.getString('toolbar.inline.label'));
      }
      catch (ex)
      {
        ucfirst(forwardInlineMenuItem);
      }
    }
    if (forwardAsAttachmentMenuItem)
    {
      forwardAsAttachmentMenuItem.setAttribute('default', (forwardType === 0));
      try
      {
        forwardAsAttachmentMenuItem.setAttribute('label', strbundle.getString('toolbar.asAttachment.label'));
      }
      catch (ex)
      {
        ucfirst(forwardAsAttachmentMenuItem);
      }
    }

    var headerForwardButton = document.getElementById('hdrForwardButton');  // Thunderbird 3.0 pre ..
    if (headerForwardButton === null)
    {
      var headerViewButtonBox = document.getElementById('msgHeaderViewDeck').selectedPanel.
                                         getElementsByTagName('header-view-button-box').item(0);  // Thunderbird 3.0a1 .. 3.0b4
      if (headerViewButtonBox !== null)
      {
        headerForwardButton = document.getAnonymousElementByAttribute(headerViewButtonBox, 'anonid', 'hdrForwardButton');
      }
    }
    if (headerForwardButton !== null)
    {
      var headerForwardButtonFirstInline = document.getElementById('hdrForwardButtonFirstInline');
      var headerForwardButtonLastInline = document.getElementById('hdrForwardButtonLastInline');
      if (headerForwardButtonFirstInline !== null) // Dropdown menuitems already exist, update if pref change
      {
        if (forwardType === 0)
        {
          if (headerForwardButtonLastInline.hasAttribute('hidden')) // Show as attachment as first item
          {
            headerForwardButtonFirstInline.setAttribute('hidden', 'true'); // by hiding first and showing last
            headerForwardButtonLastInline.removeAttribute('hidden');
          }
        }
        else
        {
          if (headerForwardButtonFirstInline.hasAttribute('hidden')) // Show inline as first item
          {
            headerForwardButtonFirstInline.removeAttribute('hidden');  // by showing first and hiding last
            headerForwardButtonLastInline.setAttribute('hidden', 'true');
          }
        }
      }
      else
      {
        var osString = Components.classes['@mozilla.org/xre/app-info;1'].getService(Components.interfaces.nsIXULRuntime).OS;
        var image = window.getComputedStyle(headerForwardButton, null).getPropertyValue('list-style-image');
        var region = window.getComputedStyle(headerForwardButton, null).getPropertyValue('-moz-image-region');
        var menupopup = headerForwardButton.getElementsByTagName('menupopup').item(0);
        var menuitem;
        headerForwardButton.setAttribute('type', 'menu-button');
        if ((osString !== 'WINNT') && (osString !== 'Linux') && (osString !== 'Darwin'))
        {
          headerForwardButton.setAttribute('class', 'msgHeaderView-button hdrReplyButton'); // Make forward button look like reply button
          headerForwardButton.setAttribute('style', 'list-style-image: ' + image + '; -moz-image-region: ' + region + ';'); // except for the icon
        }

        if (menupopup === null)
        {
          headerForwardButton.appendChild(document.createElement('menupopup'));
          menupopup = headerForwardButton.firstChild;
        }

        menuitem = menupopup.appendChild(document.createElement('menuitem'));
        menuitem.setAttribute('id', 'hdrForwardButtonFirstInline');
        menuitem.setAttribute('label', strbundle.getString('header.inline.label'));
        menuitem.setAttribute('tooltiptext', strbundle.getString('hdrForwardButton.tooltip'));
        menuitem.addEventListener('command', function(event) { MsgForwardAsInline(event); event.stopPropagation(); RestoreFocusAfterHdrButton(); }, false);
        if (forwardType === 0) { menuitem.setAttribute('hidden', 'true'); }  // Show first depending on pref

        menuitem = menupopup.appendChild(document.createElement('menuitem'));
        menuitem.setAttribute('id', 'hdrForwardButtonAsAttachment');
        menuitem.setAttribute('label', strbundle.getString('header.asAttachment.label'));
        menuitem.setAttribute('tooltiptext', strbundle.getString('hdrForwardButton.tooltip'));
        menuitem.addEventListener('command', function(event) { MsgForwardAsAttachment(event); event.stopPropagation(); RestoreFocusAfterHdrButton(); }, false);

        menuitem = menupopup.appendChild(document.createElement('menuitem'));
        menuitem.setAttribute('id', 'hdrForwardButtonLastInline');
        menuitem.setAttribute('label', strbundle.getString('header.inline.label'));
        menuitem.setAttribute('tooltiptext', strbundle.getString('hdrForwardButton.tooltip'));
        menuitem.addEventListener('command', function(event) { MsgForwardAsInline(event); event.stopPropagation(); RestoreFocusAfterHdrButton(); }, false);
        if (forwardType !== 0) { menuitem.setAttribute('hidden', 'true'); }  // Show last depending on pref
      }
    }
  },

  // context menu
  updateForwardAsContextMenu: function ()
  {
    var strbundle = document.getElementById('bundle_forward');
    var forwardAsMenu = document.getElementById('mailContext-forwardAsMenu');
    if (forwardAsMenu)
    {
      try
      {
        forwardAsMenu.setAttribute('label', strbundle.getString('forwardAsMenu.label'));
        forwardAsMenu.setAttribute('accesskey', strbundle.getString('forwardAsMenu.accesskey'));
      }
      catch (ex)
      {
      }
    }
    var forwardAsInline = document.getElementById('mailContext-forwardAsMenu-forwardAsInline');
    if (forwardAsInline)
    {
      try
      {
        forwardAsInline.setAttribute('label', strbundle.getString('forwardAsInline.label'));
        forwardAsInline.setAttribute('accesskey', strbundle.getString('forwardAsInline.accesskey'));
      }
      catch (ex)
      {
      }
    }
    var forwardAsAttachment = document.getElementById('mailContext-forwardAsMenu-forwardAsAttachment');
    if (forwardAsAttachment)
    {
      try
      {
        forwardAsAttachment.setAttribute('label', strbundle.getString('forwardAsAttachment.label'));
        forwardAsAttachment.setAttribute('accesskey', strbundle.getString('forwardAsAttachment.accesskey'));
      }
      catch (ex)
      {
      }
    }
  },

  // threadPane and messagePane menu menuitem

  SetupForwardAsMenuMenuItem: function (menuID, numSelected, forceHide)
  {
    ShowMenuItem(menuID, (numSelected <= 1) && !forceHide);
    EnableMenuItem(menuID, (numSelected === 1));
  },

  // threadPane

  forward_fillThreadPaneContextMenu: function ()
  {
    var numSelected = GetNumSelectedMessages();
    myForwardExtension.SetupForwardAsMenuMenuItem('threadPaneContext-forwardAsMenu', numSelected, false);
  },

  // messagePane

  forward_fillMessagePaneContextMenu: function ()
  {
    var message = GetLoadedMessage();
    var numSelected = (message) ? 1 : 0;

    // don't show mail items for links/images, just show related items.
    var hideMailItems = gContextMenu.onImage || gContextMenu.onLink;
    myForwardExtension.SetupForwardAsMenuMenuItem('messagePaneContext-forwardAsMenu', numSelected, (numSelected === 0 || hideMailItems));
  },

  // mail (TB3)

  forward_fillMailContextMenu: function ()
  {
    var numSelected = GetNumSelectedMessages();

    // don't show mail items for links/images, just show related items.
    var hideMailItems = gContextMenu.onImage || gContextMenu.onLink;
    myForwardExtension.SetupForwardAsMenuMenuItem('mailContext-forwardAsMenu', numSelected, (numSelected === 0 || hideMailItems));
  },

  // Event listeners

  installListeners: function (event)
  {
    var el;

    myForwardExtension.updateForwardButtonMenu();
    myForwardExtension.updateForwardAsContextMenu();

    el = document.getElementById('threadPaneContext');
    if (el)
    {
      el.addEventListener('popupshowing', myForwardExtension.forward_fillThreadPaneContextMenu, false);
    }

    el = document.getElementById('messagePaneContext');
    if (el)
    {
      el.addEventListener('popupshowing', myForwardExtension.forward_fillMessagePaneContextMenu, false);
    }

    el = document.getElementById('mailContext');
    if (el)
    {
      el.addEventListener('popupshowing', myForwardExtension.forward_fillMailContextMenu, false);
    }
  },

  uninstallListeners: function (event)
  {
    var el;
    el = document.getElementById('threadPaneContext');
    if (el)
    {
      el.removeEventListener('popupshowing', myForwardExtension.forward_fillThreadPaneContextMenu, false);
    }

    el = document.getElementById('messagePaneContext');
    if (el)
    {
      el.removeEventListener('popupshowing', myForwardExtension.forward_fillMessagePaneContextMenu, false);
    }

    el = document.getElementById('mailContext');
    if (el)
    {
      el.removeEventListener('popupshowing', myForwardExtension.forward_fillMailContextMenu, false);
    }
  },

  // Pref observer

  register: function()
  {
    var prefService = Components.classes['@mozilla.org/preferences-service;1'].
                                 getService(Components.interfaces.nsIPrefService);
    this._forward_message_mode = prefService.getBranch('mail.forward_message_mode');  // Forward mode
    this._forward_message_mode.QueryInterface(Components.interfaces.nsIPrefBranch2);
    this._forward_message_mode.addObserver('', this, false);

    this._pane_config_dynamic = prefService.getBranch('mail.pane_config.dynamic');  // Layout change
    this._pane_config_dynamic.QueryInterface(Components.interfaces.nsIPrefBranch2);
    this._pane_config_dynamic.addObserver('', this, false);
  },

  unregister: function()
  {
    if(!this._forward_message_mode)
    {
      return;
    }
    this._forward_message_mode.removeObserver('', this);
    this._pane_config_dynamic.removeObserver('', this);
  },

  observe: function(aSubject, aTopic, aData)
  {
    if(aTopic !== 'nsPref:changed')
    {
      return;
    }
    this.updateForwardButtonMenu();
  }
};

window.addEventListener('load', myForwardExtension.installListeners, false);
window.addEventListener('unload', myForwardExtension.uninstallListeners, false);

// Watch for preference changes
myForwardExtension.register();
